import sys
import logging
from odoo_tools.compat import Path
import os
from contextlib import contextmanager

_logger = logging.getLogger(__name__)

try:
    from subprocess import run as sp_run, PIPE

    def run(*args, **kwargs):
        if 'check' not in kwargs:
            kwargs['check'] = True

        ret = sp_run(*args, **kwargs)

        if kwargs.get('stdout') == PIPE:
            return ret.stdout

        return ret

except ImportError:
    from subprocess import check_call, call, PIPE, check_output

    def run(*args, **kwargs):
        if 'check' not in kwargs or kwargs['check']:
            if kwargs.get('stdout') == PIPE:
                return check_output(*args)
            else:
                return check_call(*args)
        else:
            return call(*args)


@contextmanager
def cd(directory):
    cwd = Path.cwd()
    try:
        os.chdir(directory)
        yield
    finally:
        os.chdir(str(cwd))


try:
    from tempfile import TemporaryDirectory
except ImportError:
    from tempfile import mkdtemp
    from shutil import rmtree

    from contextlib import contextmanager

    @contextmanager
    def TemporaryDirectory():
        directory = mkdtemp()
        try:
            yield directory
        finally:
            try:
                rmtree(directory)
            except Exception:
                _logger.info(
                    "Couldn't remove temporary folder %s",
                    directory,
                    exc_info=True
                )


def get_module_path(module):
    module_obj = sys.modules[module]
    return Path(module_obj.__file__).parent


def get_resource(module, path):
    module_path = get_module_path(module)
    return module_path / path


class DictObject(dict):
    def __getattr__(self, key):
        return self.get(key, False)

    def __setattr__(self, key, value):
        self[key] = value
